import { SocketParams } from './socketParams';
/**
 * websocket封装
 * dev: facebesidewyj
 * @class
 */
export default class Socket {
    private url;
    private webSocket;
    private heartBeatTimer;
    private heartBeatMsg;
    private openCallback;
    private messageCallback;
    private closeCallback;
    private errorCallback;
    private reconnectLimit;
    private isInReconnect;
    private reconnectCount;
    constructor({ url, heartBeatMsg, reconnectLimit, openCallback, messageCallback, closeCallback, errorCallback, }: SocketParams);
    /**
     * 初始化websocket
     */
    private createSocket;
    /**
     * 初始化ws事件监听
     */
    private initEventMonitor;
    /**
     * 建立连接函数
     */
    private onOpen;
    /**
     * 接收数据函数
     */
    private onMessage;
    /**
     * 关闭处理函数
     */
    private onClose;
    /**
     * 错误处理函数
     */
    private onError;
    /**
     * 手动关闭socket
     */
    close(): void;
    /**
     * 发送业务数据
     * @param {Object} msg 业务数据json对象
     */
    sendMsg(msg: any): void;
    /**
     * 重连机制
     */
    private reconnect;
    /**
     * 保证发送状态就绪
     * @param {Function} success 成功回调
     */
    private readyToSendMessage;
    /**
     * 开始心跳机制(5秒发送一次❤)
     */
    private startHeartBeat;
    /**
     * 停止心跳机制
     */
    private stopHeartBeat;
    /**
     * 重置重连次数
     */
    private resetReconnectCount;
    /**
     * 检查socket是否连接成功
     * @return {Boolean} 是否建立连接成功
     */
    private isSocketOpen;
    /**
     * 检查socket是否正在关闭
     * @return {Boolean} 是否正在关闭
     */
    private isSocketClosing;
    /**
     * 检查socket是否已经关闭
     * @return {Boolean} 是否已经关闭
     */
    private isSocketClose;
}
