"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _config = _interopRequireDefault(require("./config"));

var _observer = _interopRequireDefault(require("../observer"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * websocket封装
 * dev: facebesidewyj
 * @class
 */
var Socket = /*#__PURE__*/function () {
  function Socket(_ref) {
    var url = _ref.url,
        heartBeatMsg = _ref.heartBeatMsg,
        reconnectLimit = _ref.reconnectLimit,
        openCallback = _ref.openCallback,
        messageCallback = _ref.messageCallback,
        closeCallback = _ref.closeCallback,
        errorCallback = _ref.errorCallback;

    _classCallCheck(this, Socket);

    // 标识是否正在重连
    this.isInReconnect = false; // 重连次数

    this.reconnectCount = 0;
    this.url = url || _config["default"].DEFAULT_URL;
    this.heartBeatMsg = heartBeatMsg || _config["default"].DEFAULT_HEARTMSG;
    this.reconnectLimit = reconnectLimit || _config["default"].RECONNECT_LIMIT;
    this.openCallback = openCallback;
    this.messageCallback = messageCallback;
    this.closeCallback = closeCallback;
    this.errorCallback = errorCallback;
    this.createSocket();
  }
  /**
   * 初始化websocket
   */


  _createClass(Socket, [{
    key: "createSocket",
    value: function createSocket() {
      try {
        this.webSocket = new WebSocket(this.url);
        this.initEventMonitor();
      } catch (e) {
        console.error('WebSocket创建连接失败');
      }
    }
    /**
     * 初始化ws事件监听
     */

  }, {
    key: "initEventMonitor",
    value: function initEventMonitor() {
      this.onOpen();
      this.onMessage();
      this.onClose();
      this.onError();
    }
    /**
     * 建立连接函数
     */

  }, {
    key: "onOpen",
    value: function onOpen() {
      var _this = this;

      this.webSocket.addEventListener('open', function (res) {
        // 一旦建立成功，重置重连次数
        _this.resetReconnectCount(); // 暂停心跳


        _this.stopHeartBeat(); // 重新启动心跳请求


        _this.startHeartBeat();

        _observer["default"].pub('socketReady', []);

        if (_this.openCallback) {
          _this.openCallback(res);
        }
      });
    }
    /**
     * 接收数据函数
     */

  }, {
    key: "onMessage",
    value: function onMessage() {
      var _this2 = this;

      this.webSocket.addEventListener('message', function (res) {
        if (_this2.messageCallback && res.data) {
          _this2.messageCallback(res.data);
        }
      });
    }
    /**
     * 关闭处理函数
     */

  }, {
    key: "onClose",
    value: function onClose() {
      var _this3 = this;

      this.webSocket.addEventListener('close', function (res) {
        console.error("WebSocket\u8FDE\u63A5\u5173\u95ED\uFF0C\u72B6\u6001\u7801\uFF1A".concat(res.code));

        if (_this3.closeCallback) {
          _this3.closeCallback(res);
        }

        _this3.stopHeartBeat(); // 非正常断开并且重连次数小于等于重连上限


        if (res.code !== _config["default"].ERROR_CODE.normal && _this3.reconnectCount <= _this3.reconnectLimit) {
          _this3.reconnect();
        }
      });
    }
    /**
     * 错误处理函数
     */

  }, {
    key: "onError",
    value: function onError() {
      var _this4 = this;

      this.webSocket.addEventListener('error', function (res) {
        console.error('WebSocket in error');

        if (_this4.errorCallback) {
          _this4.errorCallback(res);
        }

        _this4.stopHeartBeat();
      });
    }
    /**
     * 手动关闭socket
     */

  }, {
    key: "close",
    value: function close() {
      this.webSocket.close();
    }
    /**
     * 发送业务数据
     * @param {Object} msg 业务数据json对象
     */

  }, {
    key: "sendMsg",
    value: function sendMsg(msg) {
      var _this5 = this;

      this.readyToSendMessage(function () {
        try {
          var json = JSON.stringify(msg);

          _this5.webSocket.send(json);
        } catch (error) {
          console.error(error);
        }
      });
    }
    /**
     * 重连机制
     */

  }, {
    key: "reconnect",
    value: function reconnect() {
      var _this6 = this;

      if (this.isInReconnect) {
        return;
      }

      this.isInReconnect = true; // 递增重试建立连接

      setTimeout(function () {
        _this6.createSocket(); // 重连次数加1


        _this6.reconnectCount++;
        _this6.isInReconnect = false;
      }, _config["default"].RECONNECT_INTERVAL * this.reconnectCount);
    }
    /**
     * 保证发送状态就绪
     * @param {Function} success 成功回调
     */

  }, {
    key: "readyToSendMessage",
    value: function readyToSendMessage(success) {
      // 发送时先判断连接是否关闭，如果已经关闭并且连接次数已经超过了上限（之前就触发过自动重连，并且在上限之内没有连接上）
      if (this.isSocketClosing() || this.isSocketClose() && this.reconnectCount > this.reconnectLimit) {
        // 重置连接次数
        this.resetReconnectCount();
        this.reconnect();
      } // 如果已经成功的建立了链接，则直接发送，否则监听websocketReady成功再发送


      if (this.isSocketOpen()) {
        if (success) {
          success();
        }
      } else {
        _observer["default"].sub('socketReady', function () {
          if (success) {
            success();
          }
        });
      }
    }
    /**
     * 开始心跳机制(5秒发送一次❤)
     */

  }, {
    key: "startHeartBeat",
    value: function startHeartBeat() {
      var _this7 = this;

      this.heartBeatTimer = setInterval(function () {
        _this7.webSocket.send(_this7.heartBeatMsg);
      }, _config["default"].HEART_INTERVAL);
    }
    /**
     * 停止心跳机制
     */

  }, {
    key: "stopHeartBeat",
    value: function stopHeartBeat() {
      clearInterval(this.heartBeatTimer);
      this.heartBeatTimer = null;
    }
    /**
     * 重置重连次数
     */

  }, {
    key: "resetReconnectCount",
    value: function resetReconnectCount() {
      this.reconnectCount = 0;
    }
    /**
     * 检查socket是否连接成功
     * @return {Boolean} 是否建立连接成功
     */

  }, {
    key: "isSocketOpen",
    value: function isSocketOpen() {
      return this.webSocket && this.webSocket.readyState === WebSocket.OPEN;
    }
    /**
     * 检查socket是否正在关闭
     * @return {Boolean} 是否正在关闭
     */

  }, {
    key: "isSocketClosing",
    value: function isSocketClosing() {
      return this.webSocket && this.webSocket.readyState === WebSocket.CLOSING;
    }
    /**
     * 检查socket是否已经关闭
     * @return {Boolean} 是否已经关闭
     */

  }, {
    key: "isSocketClose",
    value: function isSocketClose() {
      return this.webSocket && this.webSocket.readyState === WebSocket.CLOSED;
    }
  }]);

  return Socket;
}();

exports["default"] = Socket;